from collections.abc import Sequence
from typing import Any

import aioinject
from aioinject import Provider
from benchmark.dependencies import (
    RepositoryA,
    RepositoryB,
    ServiceA,
    ServiceB,
    UseCase,
    create_session,
)


providers: Sequence[Provider[Any]] = [
    aioinject.Callable(create_session),
    aioinject.Callable(RepositoryA),
    aioinject.Callable(RepositoryB),
    aioinject.Callable(ServiceA),
    aioinject.Callable(ServiceB),
    aioinject.Callable(UseCase),
]


def create_container() -> aioinject.Container:
    container = aioinject.Container()
    for provider in providers:
        container.register(provider=provider)

    return container



import time
from collections.abc import AsyncIterator
from datetime import timedelta
from typing import Annotated

import httpx
from fastapi import Depends, FastAPI

from aioinject import Inject
from aioinject.ext.fastapi import AioInjectMiddleware, inject
from benchmark.container import create_container
from benchmark.dependencies import (
    RepositoryA,
    RepositoryB,
    ServiceA,
    ServiceB,
    UseCase,
    create_session,
)
from benchmark.dto import BenchmarkResult


async def fastapi_bench(
    iterations: int,
    endpoint: str,
) -> AsyncIterator[BenchmarkResult]:
    app = FastAPI()
    app.add_middleware(AioInjectMiddleware, container=create_container())

    @app.get("/aioinject")
    @inject
    async def test_aioinject(use_case: Annotated[UseCase, Inject]) -> int:
        return await use_case.execute()

    @app.get("/depends")
    async def test_depends(use_case: Annotated[UseCase, Depends()]) -> int:
        return await use_case.execute()

    @app.get("/by-hand")
    async def test_by_hand() -> int:
        async with create_session() as session:
            use_case = UseCase(
                service_a=ServiceA(repository=RepositoryA(session=session)),
                service_b=ServiceB(repository=RepositoryB(session=session)),
            )

        return await use_case.execute()

    durations = []
    async with httpx.AsyncClient(
        app=app,
        base_url="http://test",
    ) as client:
        for _ in range(iterations):
            start = time.perf_counter()
            response = await client.get(endpoint)
            durations.append(
                timedelta(seconds=time.perf_counter() - start),
            )
            response.raise_for_status()
            assert response.content == b"42"  # noqa: S101

    yield BenchmarkResult(
        name=f"FastAPI - {endpoint}",
        durations=durations,
        iterations=iterations,
    )



import time
from collections.abc import AsyncIterator
from datetime import timedelta
from typing import Annotated

import httpx
import strawberry
from strawberry import Schema
from strawberry.asgi import GraphQL
from strawberry.schema.config import StrawberryConfig

from aioinject import Inject
from aioinject.ext.strawberry import AioInjectExtension, inject
from benchmark.container import create_container
from benchmark.dependencies import (
    RepositoryA,
    RepositoryB,
    ServiceA,
    ServiceB,
    UseCase,
    create_session,
)
from benchmark.dto import BenchmarkResult


@strawberry.type
class Query:
    @strawberry.field
    async def by_hand(self) -> int:
        async with create_session() as session:
            use_case = UseCase(
                service_a=ServiceA(repository=RepositoryA(session=session)),
                service_b=ServiceB(repository=RepositoryB(session=session)),
            )
            return await use_case.execute()

    @strawberry.field
    @inject
    async def aioinject(self, use_case: Annotated[UseCase, Inject]) -> int:
        return await use_case.execute()


async def bench_strawberry(
    iterations: int,
) -> AsyncIterator[BenchmarkResult]:
    schema = Schema(
        query=Query,
        extensions=[AioInjectExtension(create_container())],
        config=StrawberryConfig(auto_camel_case=False),
    )
    app = GraphQL[None, None](schema=schema)

    resolver_names = ["by_hand", "aioinject"]
    query = """
    query { result: RESOLVER_NAME }
    """

    async with httpx.AsyncClient(
        app=app,
        base_url="http://test",
    ) as client:
        for resolver_name in resolver_names:
            durations = []
            for _ in range(iterations):
                start = time.perf_counter()
                response = await client.post(
                    "/",
                    json={
                        "query": query.replace("RESOLVER_NAME", resolver_name),
                    },
                )
                durations.append(
                    timedelta(seconds=time.perf_counter() - start),
                )
                response.raise_for_status()
                assert response.json() == {  # noqa: S101
                    "data": {"result": 42},
                }

            yield BenchmarkResult(
                name=f"Strawberry - {resolver_name}",
                durations=durations,
                iterations=iterations,
            )



import dataclasses
import functools
import statistics
from collections.abc import Sequence
from datetime import timedelta


@dataclasses.dataclass
class BenchmarkResult:
    name: str
    iterations: int
    durations: Sequence[timedelta] = dataclasses.field(repr=False)

    @functools.cached_property
    def durations_sec(self) -> Sequence[float]:
        return sorted(t.total_seconds() for t in self.durations)

    @property
    def mean(self) -> timedelta:
        return timedelta(seconds=statistics.mean(self.durations_sec))

    @property
    def median(self) -> timedelta:
        return timedelta(seconds=statistics.median(self.durations_sec))

    def percentile(self, perc: float) -> timedelta:
        index = int(len(self.durations_sec) * perc)
        return timedelta(seconds=self.durations_sec[index])



import asyncio
import functools
from collections.abc import AsyncIterator, Sequence
from datetime import timedelta
from typing import Protocol

from benchmark.benches.fastapi import fastapi_bench
from benchmark.benches.litestar import litestar_bench
from benchmark.benches.python import (
    bench_aioinject_decorator,
    bench_aioinject_raw,
    bench_python,
)
from benchmark.benches.strawberry import bench_strawberry
from benchmark.dto import BenchmarkResult


def time_to_ms(delta: timedelta | float) -> str:
    if isinstance(delta, timedelta):
        delta = delta.total_seconds()
    return f"{delta * 1000:.3f}ms"


def format_result(result: BenchmarkResult, row_template: str) -> str:
    return row_template.format(
        result.name,
        str(result.iterations),
        time_to_ms(sum(result.durations_sec)),
        time_to_ms(result.mean),
        time_to_ms(result.median),
        time_to_ms(result.percentile(0.95)),
        time_to_ms(result.percentile(0.99)),
    )


class BenchFunction(Protocol):
    def __call__(self, iterations: int) -> AsyncIterator[BenchmarkResult]:
        ...


BENCHMARK_FUNCTIONS: Sequence[BenchFunction] = [
    bench_python,
    bench_aioinject_raw,
    bench_aioinject_decorator,
    functools.partial(litestar_bench, endpoint="/aioinject"),
    functools.partial(litestar_bench, endpoint="/litestar"),
    functools.partial(fastapi_bench, endpoint="/depends"),
    functools.partial(fastapi_bench, endpoint="/aioinject"),
    functools.partial(fastapi_bench, endpoint="/by-hand"),
    bench_strawberry,
]


async def main() -> None:
    iterations = [100, 1_000, 10_000]

    row_template = "{:25} {:10} {:10} {:10} {:10} {:10} {:10}"
    print(  # noqa: T201
        row_template.format(
            "Name",
            "iterations",
            "sum",
            "mean",
            "median",
            "p95",
            "p99",
        ),
    )

    for bench_functions in BENCHMARK_FUNCTIONS:
        for count in iterations:
            async for result in bench_functions(iterations=count):
                print(format_result(result, row_template))  # noqa: T201
        print()  # noqa: T201


if __name__ == "__main__":
    asyncio.run(main())



import time

import aioinject


class A:
    pass


class B:
    def __init__(self, a: A):
        self.a = a


class C:
    def __init__(self, b: B):
        self.b = b


def simple_factory():
    return C(b=B(a=A()))


def main():
    container = aioinject.Container()
    container.register(aioinject.Callable(A))
    container.register(aioinject.Callable(B))
    container.register(aioinject.Callable(C))

    n_rounds = 1
    n = 1_000_000
    sum_ = 0
    for _ in range(n_rounds):
        start = time.perf_counter()
        for _ in range(n):
            with container.sync_context() as ctx:
                ctx.resolve(C)
        diff = time.perf_counter() - start
        print(diff)
        sum_ += diff
    print(sum_)


main()



import time
from collections.abc import AsyncIterator
from datetime import timedelta
from typing import Annotated

from aioinject import Inject, inject
from benchmark.container import create_container
from benchmark.dependencies import (
    RepositoryA,
    RepositoryB,
    ServiceA,
    ServiceB,
    UseCase,
    create_session,
)
from benchmark.dto import BenchmarkResult


async def bench_aioinject_raw(
    iterations: int,
) -> AsyncIterator[BenchmarkResult]:
    container = create_container()

    durations = []
    for _ in range(iterations):
        start = time.perf_counter()
        async with container.context() as ctx:
            use_case = await ctx.resolve(UseCase)
            await use_case.execute()

        durations.append(
            timedelta(seconds=time.perf_counter() - start),
        )
    yield BenchmarkResult(
        iterations=iterations,
        durations=durations,
        name="Aioinject",
    )


async def bench_aioinject_decorator(
    iterations: int,
) -> AsyncIterator[BenchmarkResult]:
    container = create_container()

    durations = []

    @inject
    async def injectee(use_case: Annotated[UseCase, Inject]) -> int:
        return await use_case.execute()

    for _ in range(iterations):
        start = time.perf_counter()
        async with container.context():
            await injectee()  # type: ignore[call-arg]
        durations.append(
            timedelta(seconds=time.perf_counter() - start),
        )
    yield BenchmarkResult(
        iterations=iterations,
        durations=durations,
        name="Aioinject - Decorator",
    )


async def bench_python(iterations: int) -> AsyncIterator[BenchmarkResult]:
    durations = []
    for _ in range(iterations):
        start = time.perf_counter()
        async with create_session() as session:
            use_case = UseCase(
                service_a=ServiceA(repository=RepositoryA(session=session)),
                service_b=ServiceB(repository=RepositoryB(session=session)),
            )
            await use_case.execute()
        durations.append(
            timedelta(seconds=time.perf_counter() - start),
        )
    yield BenchmarkResult(
        iterations=iterations,
        durations=durations,
        name="Python",
    )



import time
from collections.abc import AsyncIterator
from datetime import timedelta
from typing import Annotated

import httpx
from fastapi import Depends, FastAPI
from litestar import Litestar, get
from litestar.di import Provide
from litestar.logging import LoggingConfig

from aioinject import Inject
from aioinject.ext.fastapi import AioInjectMiddleware, inject
from aioinject.ext.litestar import AioInjectPlugin
from benchmark.container import create_container
from benchmark.dependencies import (
    RepositoryA,
    RepositoryB,
    ServiceA,
    ServiceB,
    UseCase,
    create_session,
    Session,
)
from benchmark.dto import BenchmarkResult


async def _get_session() -> AsyncIterator[Session]:
    yield Session()


async def _get_repository_a(session: Session) -> RepositoryA:
    return RepositoryA(session)


async def _get_repository_b(session: Session) -> RepositoryB:
    return RepositoryB(session)


async def _get_service_a(repository_a: RepositoryA) -> ServiceA:
    return ServiceA(repository=repository_a)


async def _get_service_b(repository_b: RepositoryB) -> ServiceB:
    return ServiceB(repository=repository_b)


async def _get_usecase(
    service_a: ServiceA,
    service_b: ServiceB,
) -> UseCase:
    return UseCase(
        service_a=service_a,
        service_b=service_b,
    )


@get("/aioinject")
@inject
async def test_aioinject(use_case: Annotated[UseCase, Inject]) -> int:
    return await use_case.execute()


@get("/litestar")
async def test_litestar(use_case: UseCase) -> int:
    return await use_case.execute()


async def litestar_bench(
    iterations: int,
    endpoint: str,
) -> AsyncIterator[BenchmarkResult]:
    app = Litestar(
        plugins=[AioInjectPlugin(container=create_container())],
        route_handlers=[test_aioinject, test_litestar],
        dependencies={
            "session": Provide(_get_session),
            "repository_a": Provide(_get_repository_a),
            "repository_b": Provide(_get_repository_b),
            "service_a": Provide(_get_service_a),
            "service_b": Provide(_get_service_b),
            "use_case": Provide(_get_usecase),
        },
        logging_config=LoggingConfig(disable_existing_loggers=True),
    )

    durations = []
    async with httpx.AsyncClient(
        app=app,
        base_url="http://test",
    ) as client:
        for _ in range(iterations):
            start = time.perf_counter()
            response = await client.get(endpoint)
            durations.append(
                timedelta(seconds=time.perf_counter() - start),
            )
            response.raise_for_status()
            assert response.content == b"42"  # noqa: S101

    yield BenchmarkResult(
        name=f"Litestar - {endpoint}",
        durations=durations,
        iterations=iterations,
    )



from collections.abc import AsyncIterator, Iterator

import pytest

import aioinject
from aioinject import InjectionContext
from aioinject.context import context_var

pytestmark = [pytest.mark.anyio]

container = aioinject.Container()
container.register(aioinject.Callable(int))


@pytest.fixture
async def _container_ctx() -> AsyncIterator[InjectionContext]:
    async with container.context() as ctx:
        yield ctx


@pytest.fixture
def container_ctx(
    _container_ctx: InjectionContext,
) -> Iterator[InjectionContext]:
    token = context_var.set(_container_ctx)
    yield _container_ctx
    context_var.reset(token)


async def test_something(container_ctx: InjectionContext) -> None:
    with container.override(aioinject.Callable(str, type_=int)):
        number = await container_ctx.resolve(int)
        assert number == ""
